/*
    Copyright (c) 2006 Volker Krause <vkrause@kde.org>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "messagemodel.h"

#include <libakonadi/monitor.h>
#include <libakonadi/session.h>

#include <kmime/kmime_message.h>
#include <boost/shared_ptr.hpp>
typedef boost::shared_ptr<KMime::Message> MessagePtr;

#include <kdebug.h>
#include <klocale.h>

#include <QtCore/QDebug>

using namespace Akonadi;

class Akonadi::MessageModel::Private
{
  public:
};

MessageModel::MessageModel( QObject *parent ) :
    ItemModel( parent ),
    d( new Private() )
{
  addFetchPart( Item::PartEnvelope );
}

MessageModel::~MessageModel( )
{
  delete d;
}

int MessageModel::columnCount( const QModelIndex & parent ) const
{
  if ( !parent.isValid() )
    return 5; // keep in sync with the column type enum

  return 0;
}

QVariant MessageModel::data( const QModelIndex & index, int role ) const
{
  if ( !index.isValid() )
    return QVariant();
  if ( index.row() >= rowCount() )
    return QVariant();
  Item item = itemForIndex( index );
  if ( !item.hasPayload<MessagePtr>() )
    return QVariant();
  MessagePtr msg = item.payload<MessagePtr>();
  if ( role == Qt::DisplayRole ) {
    switch ( index.column() ) {
      case Subject:
        return msg->subject()->asUnicodeString();
      case Sender:
        return msg->from()->asUnicodeString();
      case Receiver:
        return msg->to()->asUnicodeString();
      case Date:
        return msg->date()->asUnicodeString();
      case Size:
        // TODO
        return 0; // pass modeltest
      default:
        return QVariant();
    }
  }

  return ItemModel::data( index, role );
}

QVariant MessageModel::headerData( int section, Qt::Orientation orientation, int role ) const
{
  if ( orientation == Qt::Horizontal && role == Qt::DisplayRole ) {
    switch ( section ) {
      case Subject:
        return i18nc( "@title:column, message (e.g. email) subject", "Subject" );
      case Sender:
        return i18nc( "@title:column, sender of message (e.g. email)", "Sender" );
      case Receiver:
        return i18nc( "@title:column, receiver of message (e.g. email)", "Receiver" );
      case Date:
        return i18nc( "@title:column, message (e.g. email) timestamp", "Date" );
      case Size:
        return i18nc( "@title:column, message (e.g. email) size", "Size" );
      default:
        return QString();
    }
  }
  return ItemModel::headerData( section, orientation, role );
}

#include "messagemodel.moc"
